----------------------------------------------
-- Documented function
-- @param name
-- @return void
----------------------------------------------
function extend(t, mt)
	local __mt = getmetatable(t) or {}
	for k, v in pairs(mt) do
		__mt[k] = v
	end
	return setmetatable(t, __mt)
end

----------------------------------------------
-- Documented function
-- @param name
-- @return void
----------------------------------------------
function constant(t)
	return extend(t, {
		__newindex = function(t, n, v)
			error("Attempt to change a constant", 0)
		end
	})
end

----------------------------------------------
-- Documented function
-- @param name
-- @return void
----------------------------------------------
function protected(t)
	local __t = setmetatable({}, {
		__newindex = function(t, k, v)
			if type(v) == "function" then
				rawset(t, k, function(...)
					local status, message = pcall(v, ...)
					if status then
						return message
					else
						print(message)
					end
				end)
			else
				rawset(t, k, v)
			end
		end
	})
	for k, v in pairs(t) do
		__t[k] = v
	end
	return __t
end

----------------------------------------------
-- Documented function
-- @param name
-- @return void
----------------------------------------------
function eval(script, env, eh)
	if type(script) == "function" then
		setfenv(script, env)
		return xpcall(script, eh)
	else
		local fn, err = loadstring(script)
		if fn then
			setfenv(fn, env)
			return xpcall(fn, eh)
		else
			return nil, err
		end
	end
end

----------------------------------------------
-- Serialize into string
-- @param var mixed
-- @return string
----------------------------------------------
function serialize(var)
	local t = type(var)
	if t == "boolean" then
		return var and "true" or "false"
	elseif t == "number" then
		return tostring(var)
	elseif t == "string" then
		return string.format("%q", var)
	elseif t == "table" then
		local s = "{"
		for k, v in pairs(var) do
			local t = type(k)
			if t == "number" then
				s = s .. serialize(v) .. ","
			elseif t == "string" or t == "boolean" or t == "table" then
				s = string.format("%s[%s]=%s,", s, serialize(k), serialize(v))
			end
		end
		return s .. "}"
	else
		return "{}"
	end
end

----------------------------------------------
-- Deserialize string
-- @param s string
-- @return boolean, table
----------------------------------------------
function deserialize(s)
	local success, result = eval("return " .. s, {})
	return success and result or {}
end

----------------------------------------------
-- Divide string into table of lines
-- @param s string
-- @return table
----------------------------------------------
function string.lines(s)
	local lines = {}
	for line in string.gmatch(s, "([^\n]*)\n") do
		table.insert(lines, line)
	end
	table.insert(lines, string.match(s, "\n([^\n]*)$"))
	return lines
end

----------------------------------------------
-- Trims whitespace from start and end of string
-- @param s string
-- @return string
----------------------------------------------
function string.trim(s)
	return string.match(s, "^%s*(.-)%s*$")
end

----------------------------------------------
-- Get passed cli arguments
-- @param arg string
-- @param options string
-- @return string
----------------------------------------------
function getopt(arg, options)
	local tab = {}
	for k, v in ipairs(arg) do
		if string.sub(v, 1, 2) == "--" then
			local x = string.find(v, "=", 1, true)
			if x then tab[string.sub(v, 3, x-1)] = string.sub(v, x+1)
			else tab[string.sub(v, 3)] = true
			end
		elseif string.sub(v, 1, 1) == "-" then
			local y = 2
			local l = string.len(v)
			local jopt
			while (y <= l) do
				jopt = string.sub(v, y, y)
				if string.find(options, jopt, 1, true) then
					if y < l then
						tab[jopt] = string.sub(v, y+1)
						y = l
					else
						tab[jopt] = arg[k + 1]
					end
				else
					tab[jopt] = true
				end
				y = y + 1
			end
		end
	end
	return tab
end

----------------------------------------------
-- Performs a command and returns output and exit code
-- @param cmd string
-- @return string output from command, int exit code
----------------------------------------------
function exec_command(cmd)
	local f = io.popen(cmd..'; echo "-retcode:$?"' ,'r')
	local l = f:read '*a'
	f:close()
	local i1,i2,ret = l:find('%-retcode:(%d+)\n$')
	l = l:sub(1,i1-1)
	return l, tonumber(ret)
end

----------------------------------------------
-- Deep copy table, also applies metatables
-- @param t table
-- @return table
----------------------------------------------
function deepcopy(t)
	if type(t) ~= 'table' then return t end
	local mt = getmetatable(t)
	local res = {}
	for k,v in pairs(t) do
		if type(v) == 'table' then
			v = deepcopy(v)
		end
		res[k] = v
	end
	setmetatable(res,mt)
	return res
end

----------------------------------------------
-- URL decode string
-- @param str string
-- @return string
----------------------------------------------
function url_decode(str)
	str = string.gsub (str, "+", " ")
	str = string.gsub (str, "%%(%x%x)",
		function(h) return string.char(tonumber(h,16)) end)
	str = string.gsub (str, "\r\n", "\n")
	return str
end

----------------------------------------------
-- URL encode string
-- @param str string
-- @return string
----------------------------------------------
function url_encode(str)
	if (str) then
		str = string.gsub (str, "\n", "\r\n")
		str = string.gsub (str, "([^%w%-%.%_%~ ])",
			function (c) return string.format ("%%%02X", string.byte(c)) end)
		str = string.gsub (str, " ", "+")
	end
	return str
end
